import { describe, it, expect, beforeAll } from "@jest/globals";
import StatusesService from "../src/services/StatusesService";
import {
  RECIPIENT_CLASS_NAME,
  SENDER_CLASS_NAME,
} from "../src/services/StatusesService";
import { SenderStatusObject, arrayBufferToBase64 } from "../src";

describe("StatusesService", () => {
  let statusesService;

  beforeAll(() => {
    statusesService = new StatusesService();
  });

  it("encodes/decodes sender status object", () => {
    const textEncoder = new TextEncoder(); // always utf-8
    const stringBytes = textEncoder.encode("test bytes");
    const stringBytesBase64 = arrayBufferToBase64(stringBytes);

    const senderStatusObject: SenderStatusObject = {
      signatureTime: new Date().getTime(),
      signatureTimeSignature: stringBytesBase64,
      messageSignature: stringBytesBase64,
      attachmentsSignatures: {
        attachment_1: {
          list: [
            { value: stringBytesBase64, preSignHashAlgorithm: "test" },
            { value: stringBytesBase64, preSignHashAlgorithm: "test" },
          ],
        },
        attachment_2: {
          list: [{ value: stringBytesBase64, preSignHashAlgorithm: "test" }],
        },
      },
      messageSignatureHashingAlgorithm:
        StatusesService.MIME_SIGNATURE_HASHING_ALGORITHMS.PARTS,
      senderSystem: StatusesService.EMAIL_VENDORS.GMAIL,
      pseudoPlainPartsSignature: "hhg=",
      sealHash: "hhg=",
      sealSignature: "hhg=",
    };

    const senderStatusObjectToCompare = {
      signatureTime: senderStatusObject.signatureTime,
      signatureTimeSignature: stringBytesBase64,
      messageSignature: stringBytesBase64,
      attachmentsSignatures: {
        attachment_1: {
          list: [
            { value: stringBytesBase64, preSignHashAlgorithm: "test" },
            { value: stringBytesBase64, preSignHashAlgorithm: "test" },
          ],
        },
        attachment_2: {
          list: [{ value: stringBytesBase64, preSignHashAlgorithm: "test" }],
        },
      },
      messageSignatureHashingAlgorithm:
        StatusesService.MIME_SIGNATURE_HASHING_ALGORITHMS.PARTS,
      senderSystem: StatusesService.EMAIL_VENDORS.GMAIL,
      pseudoPlainPartsSignature: "hhg=",
      sealHash: "hhg=",
      sealSignature: "hhg=",
    };

    const statusObjectBytes = statusesService.encodeSenderStatusObject(
      senderStatusObject
    );

    const decodedStatusObject = statusesService.decodeSenderStatusObject(
      statusObjectBytes
    );

    expect(decodedStatusObject).toEqual(senderStatusObjectToCompare);
  });

  it("encodes/decodes recipient status data", () => {
    const textEncoder = new TextEncoder(); // always utf-8
    const stringBytes = textEncoder.encode("test bytes");
    const stringBytesbase64 = arrayBufferToBase64(stringBytes);

    const recipientStatusData = {
      creationDate: new Date().getTime(),
      gatewayDomainHash: stringBytesbase64,
      randomToken: stringBytesbase64,
      emailHash: stringBytesbase64,
    };

    const recipientStatusDataToCompare = {
      creationDate: recipientStatusData.creationDate,
      gatewayDomainHash: stringBytesbase64,
      randomToken: stringBytesbase64,
      emailHash: stringBytesbase64,
    };

    const recipientStatusDataBytes = statusesService.encodeRecipientStatusData(
      recipientStatusData
    );

    const decodedRecipientStatusObjectData = statusesService.decodeRecipientStatusData(
      recipientStatusDataBytes
    );

    expect(decodedRecipientStatusObjectData).toEqual(
      recipientStatusDataToCompare
    );
  });

  it("encodes/decodes recipient status object", () => {
    const textEncoder = new TextEncoder(); // always utf-8
    const stringBytes = textEncoder.encode("test bytes");
    const stringBytesBase64 = arrayBufferToBase64(stringBytes);

    const recipientStatusObject = {
      iv: stringBytesBase64,
      encryptedContent: stringBytesBase64,
      decryptors: {
        decryptor_1: stringBytesBase64,
        decryptor_2: stringBytesBase64,
      },
    };

    const recipientStatusObjectBytes = statusesService.encodeRecipientStatusObject(
      recipientStatusObject
    );

    const decodedStatusObject = statusesService.decodeRecipientStatusObject(
      recipientStatusObjectBytes
    );

    const recipientStatusObjectToCompare = {
      iv: stringBytesBase64,
      encryptedContent: stringBytesBase64,
      decryptors: {
        decryptor_1: stringBytesBase64,
        decryptor_2: stringBytesBase64,
      },
    };

    expect(decodedStatusObject).toEqual(recipientStatusObjectToCompare);
  });

  it("encodes/decodes wrapper data", () => {
    const textEncoder = new TextEncoder(); // always utf-8
    const senderBytes = textEncoder.encode("test sender bytes");
    const recipientBytes = textEncoder.encode("test recipient bytes");

    const senderWrappedData = statusesService.encodeSenderWrapperData(
      arrayBufferToBase64(senderBytes)
    );
    const recipientWrappedData = statusesService.encodeRecipientWrapperData(
      recipientBytes
    );

    const senderWrappedDataDecoded = statusesService.decodeWrapperData(
      senderWrappedData
    );
    const recipientWrappedDataDecoded = statusesService.decodeWrapperData(
      recipientWrappedData
    );

    expect(senderWrappedDataDecoded).toEqual({
      data: arrayBufferToBase64(senderBytes),
      version: 1,
      className: SENDER_CLASS_NAME,
    });

    expect(recipientWrappedDataDecoded).toEqual({
      data: arrayBufferToBase64(recipientBytes),
      version: 1,
      className: RECIPIENT_CLASS_NAME,
    });
  });
});
