import { TxData, DecodedCallData } from "../../types";
import defaultRequest, { Request, Response } from "../../utils/requestAdapter";
import {
  AeternityAPIBlockData,
  AeternityAPICurrentHeight,
  AeternityAPIContractData,
  AeternityAPITxData,
  AeternityAPIBlockRange,
} from "./types";

class AeternityService {
  readonly _nodeUrls = [];
  readonly _compilerUrls = [];
  readonly _contractBytecode: string = "";
  readonly _blockHeightStep: number;

  public requestWrapper: Request = defaultRequest;

  readonly requestNode: Request;
  readonly requestCompiler: Request;

  constructor(
    nodeUrls: string[] | string,
    compilerUrls: string[] | string,
    contractBytecode: string,
    blockHeightStep?: number
  ) {
    if (!nodeUrls?.length) {
      throw new Error("Invalid node URLs");
    }

    if (!compilerUrls?.length) {
      throw new Error("Invalid compiler URLs");
    }

    if (!contractBytecode) {
      throw new Error("Aeternity contract bytecode not defined");
    }

    //Default value for block height step is 2, based on tests
    this._blockHeightStep =
      typeof blockHeightStep === "number" ? blockHeightStep : 2;
    this._nodeUrls = typeof nodeUrls === "string" ? [nodeUrls] : nodeUrls;
    this._compilerUrls =
      typeof compilerUrls === "string" ? [compilerUrls] : compilerUrls;
    this._contractBytecode = contractBytecode;

    this.requestNode = this.requestApi(this._nodeUrls);
    this.requestCompiler = this.requestApi(this._compilerUrls);
  }

  /**
   * Function provides retry mechanism if node from the array is failing to respond
   * @param apiUrls
   */
  requestApi = (apiUrls: string[]): Request => {
    let currentUrl = apiUrls[0];

    const requestWithRetry = async (config) => {
      let response;
      try {
        response = await this.requestWrapper({
          ...config,
          url: `${currentUrl}${config.url}`,
        });
      } catch (e) {
        const status = e?.response?.status;
        /**
         * Consider 400 and 404 as valid error codes.
         * Otherwise, switch to the next Aeternity node and retry
         */
        if (status === 400 || status === 404) {
          throw e;
        } else {
          const currentUrlIndex = apiUrls.indexOf(currentUrl);
          const nextUrl = apiUrls[currentUrlIndex + 1];

          if (nextUrl) {
            currentUrl = nextUrl;
            return await requestWithRetry(config);
          } else {
            throw e;
          }
        }
      }

      return response;
    };

    return requestWithRetry;
  };

  async getTxDataByHash(hash: string): Promise<TxData> {
    const { data } = (await this.requestNode({
      url: `/v2/transactions/${hash}`,
    })) as Response<AeternityAPITxData>;

    return {
      hash: data.hash,
      blockHeight: data.block_height,
      tx: {
        callData: data.tx.call_data,
      },
      blockHash: data.block_hash,
    };
  }

  async decodeContractCallData(callData: string): Promise<DecodedCallData> {
    const { data } = (await this.requestCompiler({
      url: `/decode-calldata/bytecode/`,
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      data: {
        calldata: callData,
        bytecode: this._contractBytecode,
      },
    })) as Response<AeternityAPIContractData>;

    return {
      key: data.arguments[0]?.value[0]?.key?.value,
      value: data.arguments[0]?.value[0]?.val?.value,
    };
  }

  async getMicroBlock(
    hash: string
  ): Promise<{ height: number; time: number; hash: string }> {
    const { time, height } = await this.requestNode({
      url: `/v2/micro-blocks/hash/${hash}/header`,
    }).then(({ data }: Response<AeternityAPIBlockData>) => data);

    return {
      height,
      time,
      hash,
    };
  }

  async getBlock(
    height: number
  ): Promise<{ height: number; time: number; hash: string }> {
    const { time, hash } = await this.requestNode({
      url: `/v2/key-blocks/height/${height}`,
    }).then(({ data }: Response<AeternityAPIBlockData>) => data);

    return {
      height,
      time,
      hash,
    };
  }

  async getBlockchainHeight(): Promise<number> {
    const { data } = (await this.requestNode({
      url: `/v2/key-blocks/current/height`,
    })) as Response<AeternityAPICurrentHeight>;
    return data.height;
  }

  async getBlockHeightRange(
    startingBlock: number
  ): Promise<Array<AeternityAPIBlockData>> {
    const endBlockHeight = startingBlock + this._blockHeightStep;
    const { data } = await this.requestNode({
      url: `/mdw/v2/blocks?scope=gen${startingBlock}-${endBlockHeight}`,
    }).then(({ data }: Response<AeternityAPIBlockRange>) => data);

    return data;
  }
}

export default AeternityService;
