import axios from "axios";
import { IpfsContentData, IpfsDataVersion } from "../types";
import { base64ToArrayBuffer, decompressData } from "../utils/common";
import QrCodeDataService from "./QrCodeDataService";
import CryptoService from "./CryptoService";

const GET_CONTENT_PATH = "/ipfs";

class IPFSDataRetriever {
  ipfsUrl;
  constructor(ipfsUrl: string) {
    this.ipfsUrl = ipfsUrl;
  }

  public async getDecodedContent(
    contentData: IpfsContentData,
    ipfsVersion: IpfsDataVersion
  ): Promise<ArrayBuffer> {
    const rawContent = await this.getRawContent(
      contentData.cid,
      contentData.head,
      ipfsVersion
    );
    return this.decryptContent(rawContent, contentData.key, ipfsVersion);
  }

  private async getRawContent(
    cid: string,
    head: string,
    ipfsVersion: IpfsDataVersion
  ): Promise<ArrayBuffer> {
    let assembledData;
    if (cid) {
      let ipfsFileContent;
      if (ipfsVersion === undefined) {
        const response = await axios({
          url: `${this.ipfsUrl}${GET_CONTENT_PATH}/${cid}`,
          method: "GET",
        });

        ipfsFileContent = base64ToArrayBuffer(response.data);
      } else if (ipfsVersion === "v2") {
        const result = await axios.get(
          `${this.ipfsUrl}${GET_CONTENT_PATH}/${cid}`,
          { responseType: "arraybuffer" }
        );
        ipfsFileContent = Buffer.from(result.data);
      } else {
        throw new Error("Unknown qrcode ipfs version");
      }
      assembledData = QrCodeDataService.assembleQrCodeData(
        base64ToArrayBuffer(head),
        ipfsFileContent
      );
    } else {
      assembledData = base64ToArrayBuffer(head);
    }
    return assembledData;
  }

  private async decryptContent(
    content: ArrayBuffer,
    decryptionKey: string,
    ipfsVersion: string
  ): Promise<ArrayBuffer> {
    const { key, data: iv } =
      QrCodeDataService.decodeKeyDataPair(decryptionKey);

    const decryptedZip = await CryptoService.decryptAESGCM(
      content,
      base64ToArrayBuffer(key),
      base64ToArrayBuffer(iv),
      true
    );

    if (ipfsVersion === undefined) {
      const decodedZip = new TextDecoder().decode(decryptedZip);
      return decompressData(decodedZip);
    }

    const decompressed = decompressData(decryptedZip);
    return decompressed;
  }
}

export default IPFSDataRetriever;
