import {
  RecipientStatusObject,
  RecipientStatusObjectData,
  SenderStatusObject,
  StatusObject,
  WrapperData,
} from "../types";
import { ensureUint8Array } from "../utils/common";

import { vereign } from "../generated/statuses_pb";

const SenderStatusObjectMessage = vereign.statuses.SenderStatusObject;
const RecipientStatusDataMessage = vereign.statuses.RecipientStatusData;
const RecipientStatusObjectMessage = vereign.statuses.RecipientStatusObject;
const WrapperDataMessage = vereign.statuses.WrapperData;

export const SENDER_CLASS_NAME = "SenderStatusObject";
export const RECIPIENT_CLASS_NAME = "RecipientStatusObject";
export const STATUS_FILE_PREFIX = "bs_";

export const MIME_SIGNATURE_HASHING_ALGORITHMS = {
  MIME: "MIME",
  PARTS: "PARTS",
};

export const EMAIL_VENDORS = {
  GMAIL: "GMAIL",
  OUTLOOK: "OUTLOOK",
  ROUNDCUBE: "ROUNDCUBE",
  GENERIC_MIME: "GENERIC_MIME",
};

class StatusesService {
  public static SENDER_CLASS_NAME = SENDER_CLASS_NAME;
  public static RECIPIENT_CLASS_NAME = RECIPIENT_CLASS_NAME;
  public static EMAIL_VENDORS = EMAIL_VENDORS;
  public static MIME_SIGNATURE_HASHING_ALGORITHMS = MIME_SIGNATURE_HASHING_ALGORITHMS;
  public static STATUS_FILE_PREFIX = STATUS_FILE_PREFIX;

  encodeSenderStatusObject(
    senderStatusObject: SenderStatusObject
  ): ArrayBuffer {
    const senderStatusObjectMessage = SenderStatusObjectMessage.fromObject(
      senderStatusObject
    );

    return SenderStatusObjectMessage.encode(senderStatusObjectMessage).finish();
  }

  decodeSenderStatusObject(bytes: ArrayBuffer | string): SenderStatusObject {
    const senderStatusObjectMessage = SenderStatusObjectMessage.decode(
      ensureUint8Array(bytes)
    );
    return SenderStatusObjectMessage.toObject(senderStatusObjectMessage, {
      bytes: String,
      longs: Number,
      enums: String,
    }) as SenderStatusObject;
  }

  encodeAndWrapSenderStatusObject(
    senderStatusObject: SenderStatusObject
  ): ArrayBuffer {
    const encodedSenderStatusObject = this.encodeSenderStatusObject(
      senderStatusObject
    );
    return this.encodeSenderWrapperData(encodedSenderStatusObject);
  }

  encodeRecipientStatusObject(data: RecipientStatusObject): ArrayBuffer {
    const recipientStatusObjectMessage = RecipientStatusObjectMessage.fromObject(
      data
    );

    return RecipientStatusObjectMessage.encode(
      recipientStatusObjectMessage
    ).finish();
  }

  decodeRecipientStatusObject(
    bytes: string | ArrayBuffer
  ): RecipientStatusObject {
    const recipientStatusObjectMessage = RecipientStatusObjectMessage.decode(
      ensureUint8Array(bytes)
    );
    return RecipientStatusObjectMessage.toObject(recipientStatusObjectMessage, {
      bytes: String,
      longs: Number,
    }) as RecipientStatusObject;
  }

  encodeAndWrapRecipientStatusObject(data: RecipientStatusObject): ArrayBuffer {
    const encodedSenderStatusObject = this.encodeRecipientStatusObject(data);
    return this.encodeRecipientWrapperData(encodedSenderStatusObject);
  }

  encodeRecipientStatusData(data: RecipientStatusObjectData): ArrayBuffer {
    const recipientStatusDataMessage = RecipientStatusDataMessage.fromObject(
      data
    );

    return RecipientStatusDataMessage.encode(
      recipientStatusDataMessage
    ).finish();
  }

  decodeRecipientStatusData(bytes: ArrayBuffer): RecipientStatusObjectData {
    const recipientStatusDataMessage = RecipientStatusDataMessage.decode(
      ensureUint8Array(bytes)
    );
    return RecipientStatusDataMessage.toObject(recipientStatusDataMessage, {
      bytes: String,
      longs: Number,
    }) as RecipientStatusObjectData;
  }

  encodeSenderWrapperData(
    statusObjectBytes: ArrayBuffer | string
  ): ArrayBuffer {
    return this.encodeWrapperData(statusObjectBytes, SENDER_CLASS_NAME, 1);
  }

  encodeRecipientWrapperData(
    statusObjectBytes: ArrayBuffer | string
  ): ArrayBuffer {
    return this.encodeWrapperData(statusObjectBytes, RECIPIENT_CLASS_NAME, 1);
  }

  encodeWrapperData(
    data: ArrayBuffer | string,
    className: string,
    version = 1
  ): ArrayBuffer {
    const wrapperDataMessage = WrapperDataMessage.fromObject({
      data,
      className,
      version,
    });

    return WrapperDataMessage.encode(wrapperDataMessage).finish();
  }

  decodeWrapperData(bytes: ArrayBuffer | string): WrapperData {
    const wrappedDataMessage = WrapperDataMessage.decode(
      ensureUint8Array(bytes)
    );
    return WrapperDataMessage.toObject(wrappedDataMessage, {
      bytes: String,
    }) as WrapperData;
  }

  decodeStatusObject(
    wrapperDataBytes: ArrayBuffer | string
  ): { className: string; statusObject: StatusObject } {
    const decodedWrapperData = this.decodeWrapperData(wrapperDataBytes);

    if (decodedWrapperData.className === SENDER_CLASS_NAME) {
      return {
        className: SENDER_CLASS_NAME,
        statusObject: this.decodeSenderStatusObject(decodedWrapperData.data),
      };
    } else if (decodedWrapperData.className === RECIPIENT_CLASS_NAME) {
      return {
        className: RECIPIENT_CLASS_NAME,
        statusObject: this.decodeRecipientStatusObject(decodedWrapperData.data),
      };
    } else {
      return null;
    }
  }
}

export default StatusesService;
